function [P]=int_rad_poly_Floquet(x,tx,nu,r_gamma,m_gamma,beta)

%%% Inputs: 
%%% 1) x = (lambda,a), where
%%%    - lambda is a Floquet multiplier
%%%    - a = (a_1,a_2,a_3,a_4) where a_j in C^(2m-1) is the 
%%%           vector of Fourier coefficients of v_j (j=1,2,3,4)
%%%      Note that x in \C \times C^(4(2m-1)) = C^(8m-3) 
%%% 2) tx = (tL,ta), where
%%%    - tL is the frequency of the periodic orbit
%%%    - ta is the vector of Fourier coefficients of gamma
%%%      Note that x in \R \times C^(2m-1)
%%% 3) nu = the geometric decay rate (intval quantity)
%%% 4) r_gamma = the radius about gamma from the other proof
%%% 5) m_gamma = # of Fourier modes of gamma
%%% 6) beta = the weight in the Banach space X^beta
%%%
%%% Output: P=[p0;p1;p2;p3;p4] the five radii polynomials

m=(length(x)+3)/8;

%%% Data from the periodic orbit %%%

tL = intval(tx(1));
ta = intval(tx(2:end));

i1=(2:2*m);
i2=(2*m+1:4*m-1);
i3=(4*m:6*m-2);
i4=(6*m-1:8*m-3);

%%% Floquet data %%% 

lambda = intval(x(1)); % Floquet multiplier
a1 = intval(x(i1));

%%% Computation of the apprimate inverse

[DF]=DF_Floquet_blow_up(x,tx);

A=intval(inv(DF)); 

%%%%%%%%%
%%% Y %%%
%%%%%%%%%

k=intval(-m+1:m-1)';
ba_gamma_nu_norm=sum(abs(ta).*nu.^abs(k));
a1_nu_norm=sum(abs(a1).*nu.^abs(k));

% We only evaluate F with entries |k| \le 3*m_gamma-2
[F]=int_F_Floquet_blow_up(intval(padding_Floquet(x,-(m-3*m_gamma+2))),intval(padding(tx,-(m-3*m_gamma+2))));
m_temp=3*m_gamma-2;
F0_temp = F(1);
F1_temp = F(2:2*m_temp);
F2_temp = F(2*m_temp+1:4*m_temp-1);
F3_temp = F(4*m_temp:6*m_temp-2);
F4_temp = F(6*m_temp-1:8*m_temp-3);

F1_temp_ext=padding(F1_temp,m-3*m_gamma+2);
F2_temp_ext=padding(F2_temp,m-3*m_gamma+2);
F3_temp_ext=padding(F3_temp,m-3*m_gamma+2);
F4_temp_ext=padding(F4_temp,m-3*m_gamma+2);

F=[F0_temp;F1_temp_ext;F2_temp_ext;F3_temp_ext;F4_temp_ext];

yF=A*F;

Y0=abs(yF(1));

y1 = yF(i1);
y2 = yF(i2);
y3 = yF(i3);
y4 = yF(i4);

Y1=sum(abs(y1).*nu.^abs(k));
Y2=sum(abs(y2).*nu.^abs(k));
Y3=sum(abs(y3).*nu.^abs(k));

Y4=sum(abs(y4).*nu.^abs(k))+(6/(tL*m))*ba_gamma_nu_norm*a1_nu_norm*r_gamma+(3/(tL*m))*a1_nu_norm*r_gamma^2;

Y=sup([Y0;Y1;Y2;Y3;Y4]);

%%%%%%%%%%%%%%%%%%%
%%% Z0, Z1 & Z2 %%%
%%%%%%%%%%%%%%%%%%%

%%% Z0 %%%

[iDF]=int_DF_Floquet_blow_up(intval(x),intval(tx));

nu_power = nu.^abs(intval(-m+1:m-1));

B = abs(intval(eye(8*m-3))-A*iDF);

Z0=zeros(5,1);

norm_Ba1_0=intval(max(sup(B(1,i1)./nu_power)));
norm_Ba2_0=intval(max(sup(B(1,i2)./nu_power)));
norm_Ba3_0=intval(max(sup(B(1,i3)./nu_power)));
norm_Ba4_0=intval(max(sup(B(1,i4)./nu_power)));

Z0(1)=sup(B(1,1)+norm_Ba1_0+norm_Ba2_0+norm_Ba3_0+norm_Ba4_0);

norm_B_lbda_1=sum(B(i1,1).*nu_power');
norm_B_lbda_2=sum(B(i2,1).*nu_power');
norm_B_lbda_3=sum(B(i3,1).*nu_power');
norm_B_lbda_4=sum(B(i4,1).*nu_power');

norm_B_a1_1=intval(max(sup(sum(B(i1,i1).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a2_1=intval(max(sup(sum(B(i1,i2).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a3_1=intval(max(sup(sum(B(i1,i3).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a4_1=intval(max(sup(sum(B(i1,i4).*repmat(nu_power,2*m-1,1)')./nu_power)));

norm_B_a1_2=intval(max(sup(sum(B(i2,i1).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a2_2=intval(max(sup(sum(B(i2,i2).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a3_2=intval(max(sup(sum(B(i2,i2).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a4_2=intval(max(sup(sum(B(i2,i2).*repmat(nu_power,2*m-1,1)')./nu_power)));

norm_B_a1_3=intval(max(sup(sum(B(i3,i1).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a2_3=intval(max(sup(sum(B(i3,i2).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a3_3=intval(max(sup(sum(B(i3,i2).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a4_3=intval(max(sup(sum(B(i3,i2).*repmat(nu_power,2*m-1,1)')./nu_power)));

norm_B_a1_4=intval(max(sup(sum(B(i4,i1).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a2_4=intval(max(sup(sum(B(i4,i2).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a3_4=intval(max(sup(sum(B(i4,i3).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_B_a4_4=intval(max(sup(sum(B(i4,i4).*repmat(nu_power,2*m-1,1)')./nu_power)));

Z0(2)=sup(norm_B_lbda_1+norm_B_a1_1+norm_B_a2_1+norm_B_a3_1+norm_B_a4_1);
Z0(3)=sup(norm_B_lbda_2+norm_B_a1_2+norm_B_a2_2+norm_B_a3_2+norm_B_a4_2);
Z0(4)=sup(norm_B_lbda_3+norm_B_a1_3+norm_B_a2_3+norm_B_a3_3+norm_B_a4_3);
Z0(5)=sup(norm_B_lbda_4+norm_B_a1_4+norm_B_a2_4+norm_B_a3_4+norm_B_a4_4);

%%% Z1 & Z2 %%%

Z1=zeros(5,1);
Z2=zeros(5,1);

norm_Aa1_0=intval(max(sup(abs(A(1,i1))./nu_power)));
norm_Aa2_0=intval(max(sup(abs(A(1,i2))./nu_power)));
norm_Aa3_0=intval(max(sup(abs(A(1,i3))./nu_power)));
norm_Aa4_0=intval(max(sup(abs(A(1,i4))./nu_power)));

ta_ext = intval([zeros(2*m_gamma-2,1);abs(ta);zeros(2*m_gamma-2,1)]);
omega_I = [nu.^-intval(m+2*m_gamma-3:-1:m)';intval(zeros(2*m-1,1));nu.^-intval(m:m+2*m_gamma-3)'];
ta2_omega_I=int_cubic_sumFFTcomplex(ta_ext,ta_ext,omega_I);
k0=m+2*m_gamma-2;
z11_temp=abs(ta2_omega_I(k0-m+1:k0+m-1));

z0_bound=(6+2/beta)*((nu+2)/nu)^2;

Z1(1)=sup(3*abs(A(1,i4))*z11_temp+6*norm_Aa4_0*ba_gamma_nu_norm*r_gamma+3*norm_Aa4_0*r_gamma^2);
Z2(1)=sup(abs(A(1,1))*z0_bound+2*(norm_Aa1_0+norm_Aa2_0+norm_Aa3_0+(1/beta)*norm_Aa4_0));

norm_A_lbda_1 = intval(max(sup(abs(A(i1,1)).*nu_power')));
norm_A_lbda_2 = intval(max(sup(abs(A(i2,1)).*nu_power')));
norm_A_lbda_3 = intval(max(sup(abs(A(i3,1)).*nu_power')));
norm_A_lbda_4 = intval(max(sup(abs(A(i4,1)).*nu_power')));

norm_A_a1_1=intval(max(sup(sum(abs(A(i1,i1)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a2_1=intval(max(sup(sum(abs(A(i1,i2)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a3_1=intval(max(sup(sum(abs(A(i1,i3)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a4_1=intval(max(sup(sum(abs(A(i1,i4)).*repmat(nu_power,2*m-1,1)')./nu_power)));

norm_A_a1_2=intval(max(sup(sum(abs(A(i2,i1)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a2_2=intval(max(sup(sum(abs(A(i2,i2)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a3_2=intval(max(sup(sum(abs(A(i2,i3)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a4_2=intval(max(sup(sum(abs(A(i2,i4)).*repmat(nu_power,2*m-1,1)')./nu_power)));

norm_A_a1_3=intval(max(sup(sum(abs(A(i3,i1)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a2_3=intval(max(sup(sum(abs(A(i3,i2)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a3_3=intval(max(sup(sum(abs(A(i3,i3)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a4_3=intval(max(sup(sum(abs(A(i3,i4)).*repmat(nu_power,2*m-1,1)')./nu_power)));

norm_A_a1_4=intval(max(sup(sum(abs(A(i4,i1)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a2_4=intval(max(sup(sum(abs(A(i4,i2)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a3_4=intval(max(sup(sum(abs(A(i4,i3)).*repmat(nu_power,2*m-1,1)')./nu_power)));
norm_A_a4_4=intval(max(sup(sum(abs(A(i4,i4)).*repmat(nu_power,2*m-1,1)')./nu_power)));

Z11_temp=3*sum((abs(A(i1,i4))*z11_temp).*nu_power');
Z12_temp=3*sum((abs(A(i2,i4))*z11_temp).*nu_power');
Z13_temp=3*sum((abs(A(i3,i4))*z11_temp).*nu_power');
Z14_temp=3*sum((abs(A(i4,i4))*z11_temp).*nu_power');

Z1(2)=sup((abs(lambda)+1)/(tL*m)+Z11_temp+6*norm_A_a4_1*ba_gamma_nu_norm*r_gamma+3*norm_A_a4_1*r_gamma^2);
Z2(2)=sup(norm_A_lbda_1*z0_bound+2*(norm_A_a1_1+norm_A_a2_1+norm_A_a3_1+(1/beta)*norm_A_a4_1));

Z1(3)=sup((abs(lambda)+1)/(tL*m)+Z12_temp+6*norm_A_a4_2*ba_gamma_nu_norm*r_gamma+3*norm_A_a4_2*r_gamma^2);
Z2(3)=sup(norm_A_lbda_2*z0_bound+2*(norm_A_a1_2+norm_A_a2_2+norm_A_a3_2+(1/beta)*norm_A_a4_2));

Z1(4)=sup((abs(lambda)+1/beta)/(tL*m)+Z13_temp+6*norm_A_a4_3*ba_gamma_nu_norm*r_gamma+3*norm_A_a4_3*r_gamma^2);
Z2(4)=sup(norm_A_lbda_3*z0_bound+2*(norm_A_a1_3+norm_A_a2_3+norm_A_a3_3+(1/beta)*norm_A_a4_3));

z14_1=(abs(lambda)/beta+120+154+71+14/beta)/(tL*m);
z14_2=6*norm_A_a4_4*ba_gamma_nu_norm*r_gamma+3*norm_A_a4_4*r_gamma^2;
z14_3=(ba_gamma_nu_norm^2+2*ba_gamma_nu_norm*r_gamma+r_gamma^2)/(tL*m);

Z1(5)=sup(z14_1+Z14_temp+z14_2+z14_3);
Z2(5)=sup(norm_A_lbda_4*z0_bound+2*(norm_A_a1_4+norm_A_a2_4+norm_A_a3_4+(1/beta)*norm_A_a4_4));

%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Radii Polynomials %%%
%%%%%%%%%%%%%%%%%%%%%%%%%

rad=intval([1;1;1;1;1/beta]);

P = sup([Z2 Z1+Z0-rad Y]);

end